from __future__ import absolute_import, unicode_literals

from django.core.mail import EmailMessage
from django.utils import six


# Emulate apply_async, in the case that celery isn't installed
def shared_task(func):
    def apply_async(args=None, kwargs=None, task_id=None, producer=None, link=None, link_error=None,
                    shadow=None, **options):
        if not args:
            args = []
        if not kwargs:
            kwargs = {}
        return func(*args, **kwargs)
    func.apply_async = apply_async
    return func


# https://djrill.readthedocs.org/en/v1.4/usage/sending_mail/#mandrill-specific-options
VALID_DJRILL_OPTIONS = [
    'important', 'track_opens', 'track_clicks', 'auto_text', 'auto_html', 'inline_css', 'url_strip_qs',
    'preserve_recipients', 'view_content_link', 'tracking_domain', 'signing_domain', 'return_path_domain',
    'merge_language', 'tags', 'subaccount', 'google_analytics_domain', 'google_analytics_campaign', 'metadata',
    'recipient_metadata', 'async', 'ip_pool', 'send_at'
]


@shared_task
def send_mandrill_template_mail(template_name, to, global_merge_vars=None, merge_vars=None, content_blocks=None,
                                subject=None, from_email=None, **options):
    if isinstance(to, six.string_types):
        to = [to]
    msg = EmailMessage(subject=subject, from_email=from_email, to=to)
    msg.template_name = template_name
    if content_blocks:
        msg.template_content = content_blocks
    if global_merge_vars:
        msg.global_merge_vars = global_merge_vars
    if merge_vars:
        msg.merge_vars = merge_vars
    for option, value in six.iteritems(options):
        if option not in VALID_DJRILL_OPTIONS:
            raise ValueError('Invalid option for Mandrill template: %s' % option)
        setattr(msg, option, value)
    if not subject:
        msg.use_template_subject = True
    if not from_email:
        msg.use_template_from = True
    msg.send()



from __future__ import absolute_import, unicode_literals

from allauth.account.adapter import DefaultAccountAdapter
from django.conf import settings
from django.utils import six

from .tasks import send_mandrill_template_mail

import logging

logger = logging.getLogger(__name__)

DJRILL_TEMPLATE_MAP = {
    'account/email/email_confirmation': 'email_confirmation',
    'account/email/email_confirmation_welcome': 'email_confirmation_welcome',
    'account/email/email_confirmation_signup': 'email_confirmation_signup',
    'account/email/password_reset_key': 'password_reset_key',
}


class DjrillAccountAdapter(DefaultAccountAdapter):

    def __init__(self, *args, **kwargs):
        template_map = DJRILL_TEMPLATE_MAP
        overrides = getattr(settings, 'ALLAUTH_MANDRILL_TEMPLATES', {})
        template_map.update(overrides)
        self._template_map = template_map
        super(DjrillAccountAdapter, self).__init__(*args, **kwargs)

    def send_mail(self, template_prefix, email, context):
        # Allow for overriding with '' or None
        if template_prefix in self._template_map and self._template_map[template_prefix]:
            from_email = getattr(settings, 'DEFAULT_FROM_EMAIL', "")
            mandrill_template = self._template_map[template_prefix]
            merge_vars = {}
            for key, value in six.iteritems(context):
                if key in ['current_site', 'request']:
                    # This will always be in the context, but we don't want to pass it along
                    continue
                if key == 'user':
                    value = str(value)
                elif isinstance(value, (int, str)):
                    pass
                else:
                    raise TypeError('Unknown object in allauth email context %s: %s' % (key, value))
                merge_vars[key] = value
            send_mandrill_template_mail.apply_async(kwargs={
                'template_name': mandrill_template,
                'to': email,
                'global_merge_vars': merge_vars,
                'from_email': from_email
            })
            logger.debug("Data sent: template: {} to: {} global_merge_vars: {} from_email: {}"
                         .format(mandrill_template, email, merge_vars, from_email))

        else:
            super(DjrillAccountAdapter, self).send_mail(template_prefix, email, context)



from setuptools import find_packages, setup


VERSION = __import__('allauth_djrill').__version__
github_url = 'https://github.com/xncbf/django-allauth-mandrill'

setup(
    name='django-allauth-mandrill',
    version=VERSION,
    url=github_url,
    license='Apache',
    description='Django Allauth account adapter for sending email using Mandrill templates',
    long_description=open('README.rst').read(),
    keywords='django, allauth, mailchimp, mandrill, email, djrill',
    author='Francis Kim<xncbf12@gmail.com>',
    author_email='xncbf12@gmail.com',
    zip_safe=False,
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'django>=1.3',
        'djrill>=1.4.0',
        'django-allauth>=0.20.0'
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Framework :: Django',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Operating System :: OS Independent',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Environment :: Web Environment',

    ]
)
